#!/usr/bin/python
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals
from builtins import open
from future import standard_library
standard_library.install_aliases()
import optparse
import sys
from croc import (
    SampleCurves,
    ROC,
    SlantedAC,
    CeilingAC,
    FloorAC,
    Curve,
    ScoredData,
    Linear,
    Exponential,
    Logarithm,
    Power,
)


def main(argv):
    parser = optparse.OptionParser(
        "%prog [options] < input.scoreddata > output.curve 2> info"
    )
    parser.add_option(
        "-m",
        "--tie_mode",
        type="str",
        dest="tie_mode",
        help="tie resolution method {sample,smooth,ignore}.",
        default="smooth",
    )
    parser.add_option(
        "-c",
        "--curve_type",
        type="str",
        dest="curve_type",
        help="curve type {roc,ac,slantedac,ceilingac,floorac}.",
        default="roc",
    )
    parser.add_option(
        "-t",
        "--transform",
        type="str",
        dest="transform",
        help="a python code snipit which evals to a Transform object (DEFAULT = Linear()) typical constructors include Linear, Power, Logarithm, and Exponential.",
        default="Linear()",
    )
    parser.add_option(
        "-b",
        "--best",
        type="str",
        dest="best_file",
        help="file to store the best possible performance curve",
        default=None,
    )
    parser.add_option(
        "-w",
        "--worst",
        type="str",
        dest="worst_file",
        help="file to store the worst possible performance curve",
        default=None,
    )
    parser.add_option(
        "-r",
        "--random",
        type="str",
        dest="random_file",
        help="file to store the random performance curve",
        default=None,
    )
    parser.add_option(
        "-s",
        "--samples",
        type=int,
        dest="samples",
        help="the number of times to sample ties (with 'sample' tie-mode) and the random curve (Default=500)",
        default=500,
    )
    parser.add_option(
        "--r0",
        action="store_true",
        dest="r0",
        help="Input is a rank file, indexed from 0 (Default is Scored-Label format).",
    )
    parser.add_option(
        "--r1",
        action="store_true",
        dest="r1",
        help="Input is a rank file, indexed from 1 (Default is Scored-Label format).",
    )

    (options, args) = parser.parse_args(argv)

    assert options.tie_mode in ["sample", "smooth", "ignore"]
    assert options.curve_type in ["roc", "slantedac", "ceilingac", "floorac", "ac"]
    assert options.transform != None
    assert not (options.r0 and options.r1)

    options.transform = eval(options.transform)

    rS = ScoredData.read_from_file
    if options.r0:
        rS = ScoredData.read_from_file_ranks0
    if options.r1:
        rS = ScoredData.read_from_file_ranks1

    S = rS(sys.stdin)

    if options.curve_type == "roc":
        M = ROC
    elif options.curve_type == "ac":
        M = SlantedAC
    elif options.curve_type == "slantedac":
        M = SlantedAC
    elif options.curve_type == "celingac":
        M = CeilingAC
    elif options.curve_type == "floorac":
        M = FloorAC

    F = lambda sweep: M(sweep).transform(options.transform)

    if options.tie_mode == "sample":
        curve, average, std_deviation = SampleCurves(
            lambda: F(S.sweep_threshold(tie_mode="sample")), options.samples
        )
        std_error = std_deviation / (options.samples**0.5)

        curve.write_to_file(sys.stdout)
        print("Area Under Curve (average) = ", average, file=sys.stderr)
        print(
            "Area Under Curve (standard deviation) = ", std_deviation, file=sys.stderr
        )
        print("Area Under Curve (standard error) = ", std_error, file=sys.stderr)
    else:
        C = F(S.sweep_threshold(tie_mode=options.tie_mode))
        C.write_to_file(sys.stdout)
        print("Area Under Curve = ", C.area(), file=sys.stderr)

    if options.best_file:
        file = open(options.best_file, "w")
        C = F(S.sweep_threshold_best())
        C.write_to_file(file)
        print("Area Under Best Curve = ", C.area(), file=sys.stderr)

    if options.worst_file:
        file = open(options.worst_file, "w")
        C = F(S.sweep_threshold_worst())
        C.write_to_file(file)
        print("Area Under Worst Curve = ", C.area(), file=sys.stderr)

    if options.random_file:
        curve, average, std_deviation = SampleCurves(
            lambda: F(S.sweep_threshold_random()), options.samples
        )
        std_error = std_deviation / (options.samples**0.5)

        file = open(options.random_file, "w")
        curve.write_to_file(file)
        print("Area Under Random Curve (average) = ", average, file=sys.stderr)
        print(
            "Area Under Random Curve (standard deviation) = ",
            std_deviation,
            file=sys.stderr,
        )
        print("Area Under Random Curve (standard error) = ", std_error, file=sys.stderr)


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/python
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from __future__ import unicode_literals
from builtins import open
from future import standard_library
standard_library.install_aliases()
import optparse
import sys
from croc import SampleCurves, BEDROC, CeilingAC, Curve, ScoredData, Exponential


def main(argv):
    parser = optparse.OptionParser(
        "%prog [options] < input.scoreddata > output.curve 2> info"
    )
    parser.add_option(
        "-a",
        "--alpha",
        type="float",
        dest="alpha",
        help="The alpha used to parameterize the BEDROC curve (DEFAULT=20.0)",
        default=20.0,
    )
    parser.add_option(
        "-b",
        "--best",
        type="str",
        dest="best_file",
        help="file to store the best possible performance curve",
        default=None,
    )
    parser.add_option(
        "-w",
        "--worst",
        type="str",
        dest="worst_file",
        help="file to store the worst possible performance curve",
        default=None,
    )
    parser.add_option(
        "-r",
        "--random",
        type="str",
        dest="random_file",
        help="file to store the random performance curve",
        default=None,
    )
    parser.add_option(
        "-s",
        type="int",
        dest="samples",
        help="the number of times to sample the random curve (Default=500)",
        default=500,
    )

    (options, args) = parser.parse_args(argv)

    assert options.alpha > 0

    S = ScoredData.read_from_file(sys.stdin)
    F = lambda sweep: CeilingAC(sweep).transform(Exponential(options.alpha))

    RESULTS = BEDROC(S, options.alpha)

    area = RESULTS["area"]
    curve = RESULTS["curve"]
    Barea = RESULTS["max_area"]
    Bcurve = RESULTS["max_curve"]
    Warea = RESULTS["min_area"]
    Wcurve = RESULTS["min_curve"]
    bedroc = RESULTS["BEDROC"]

    print("Area Under Curve = ", area, file=sys.stderr)
    print("Area Under Best Curve = ", Barea, file=sys.stderr)
    print("Area Under Worst Curve = ", Warea, file=sys.stderr)
    print("BEDROC = ", bedroc, file=sys.stderr)

    if options.best_file:
        file = open(options.best_file, "w")
        Bcurve.write_to_file(file)

    if options.worst_file:
        file = open(options.worst_file, "w")
        Wcurve.write_to_file(file)

    if options.random_file:
        curve, average, std_deviation = SampleCurves(
            lambda: F(S.sweep_threshold_random()), options.samples
        )
        file = open(options.random_file, "w")
        curve.write_to_file(file)
        std_error = std_deviation / (options.samples**0.5)

        print("Area Under Random Curve (average) = ", average, file=sys.stderr)
        print(
            "Area Under Random Curve (standard deviation) = ",
            std_deviation,
            file=sys.stderr,
        )
        print("Area Under Random Curve (standard error) = ", std_error, file=sys.stderr)

        print(
            "Random BEDROC (average) = ",
            (average - Warea) / (Barea - Warea),
            file=sys.stderr,
        )
        print(
            "Random BEDROC (standard deviation) = ",
            std_deviation / (Barea - Warea),
            file=sys.stderr,
        )
        print(
            "Random BEDROC (standard error) = ",
            std_error / (Barea - Warea),
            file=sys.stderr,
        )


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/python
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from builtins import open
from future import standard_library
standard_library.install_aliases()
import optparse
import sys
from croc import Curve


def main(argv):
    parser = optparse.OptionParser("%prog [options] in1.curve in2.curve > ave.curve")

    (options, args) = parser.parse_args(argv)

    if len(args) == 0:
        parser.print_help()
        sys.exit()

    curves = [Curve.read_from_file(open(FILE, "r")) for FILE in args]
    Curve.average(curves).write_to_file(sys.stdout)


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/python
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
import optparse
import sys
from croc import Curve


def main(argv):
    parser = optparse.OptionParser("%prog [options] < in.curve > out.area")
    (options, args) = parser.parse_args(argv)

    C = Curve.read_from_file(sys.stdin)
    print(C.area())


if __name__ == "__main__":
    main(sys.argv[1:])



import os
import time
import sys

from distutils.core import setup


# Loads _version.py module without importing the whole package.
def get_version_and_cmdclass(pkg_path):
    import os
    from importlib.util import module_from_spec, spec_from_file_location

    spec = spec_from_file_location(
        "version",
        os.path.join(pkg_path, "_version.py"),
    )
    module = module_from_spec(spec)  
    spec.loader.exec_module(module)  
    return module.__version__, module.get_cmdclass(pkg_path)


version, cmdclass = get_version_and_cmdclass("croc")


setup(name="CROC",
    version=version,
    description="A package for calculating ROC curves and Concentrated ROC (CROC) curves.",
    long_description="""================
The CROC Package
================

A package for calculating ROC curves and Concentrated ROC (CROC) curves written by `Dr. S. Joshua Swamidass <http://swami.wustl.edu>`_.

Citation
--------

  | **A CROC Stronger than ROC: Measuring, Visualizing, and Optimizing Early Retrieval**
  | S. Joshua Swamidass, Chloe-Agathe Azencott, Kenny Daily and Pierre Baldi
  | *Bioinformatics*, April 2010, `doi:10.1093/bioinformatics/btq140 <http://bioinformatics.oxfordjournals.org/cgi/content/abstract/btq140>`_

Description
-----------

This pure-python package is designed to be a standardized implementation of performance curves
and metrics for use either in python scripts or through a simple commandline interface. As a standardized implementation
its output is robust enough to be using in publishable scientific work.

With this package, one can easily:

#. Compute the coordinates of both Accumulation Curves and ROC curves.
#. Handle ties appropriately using several methods.
#. Compute the BEDROC metric.
#. Vertically add and average the performance curves of several cross-validation folds.
#. Focus on the early part of the ROC curve by using several x-axis transforms.

The docstrings in this module are fairly complete and the scripts provide simple access to
the most common functions. 
""",
    author="S. Joshua Swamidass",
    url="http://swami.wustl.edu/CROC",
    author_email="swamidass@gmail.com",
    install_requires=["future"],
    classifiers=["Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Science/Research",
        "License :: Free for non-commercial use",
        "Natural Language :: English",
        "Environment :: Console",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        "Topic :: Scientific/Engineering :: Visualization"],
    scripts=['scripts/croc-average', 'scripts/croc-bedroc', 'scripts/croc-area', 'scripts/croc-curve'],
    packages=['croc']
)



# -*- coding: utf-8 -*-
# This file is part of 'miniver': https://github.com/jbweston/miniver
#
# This file will be overwritten by setup.py when a source or binary
# distribution is made.  The magic value "__use_git__" is interpreted by
# _version.py.

version = "__use_git__"

# These values are only set if the distribution was created with 'git archive'
refnames = "$Format:%D$"
git_hash = "$Format:%h$"
